import requests, re
from bs4 import BeautifulSoup

from .serviceendpoint import ServiceEndpoint
from pyews.utils.exchangeversion import ExchangeVersion
from pyews.utils.exceptions import ObjectType, DeleteTypeError, SoapResponseHasError, SoapAccessDeniedError


class DeleteItem(ServiceEndpoint):
    '''Child class of :doc:`serviceendpoint` which deletes items (typically email messages) from a users mailboxes.
    
    Examples:
        To use any service class you must provide a :doc:`../configuration/userconfiguration` object first.
        Like all service classes, you can access formatted properties from the EWS endpoint using the `response` property.
        
        If you want to move a single message to the `Deleted Items` folder then provide a string value of the message ID.  The default `delete_type` is to move a message to the `Deleted Items` folder.
            
        .. code-block:: python

           userConfig = UserConfiguration(
               'first.last@company.com',
               'mypassword123'
           )
        
           messageId = 'AAMkAGZjOTlkOWExLTM2MDEtNGI3MS04ZDJiLTllNzgwNDQxMThmMABGAAAAAABdQG8UG7qjTKf0wCVbqLyMBwC6DuFzUH4qRojG/OZVoLCfAAAAAAEMAAC6DuFzUH4qRojG/OZVoLCfAAAu4Y9UAAA='

           deleteItem = DeleteItem(messageId, userConfig)

        If you want to HardDelete a single message then provide a string value of the message ID and specify the `delete_type` as `HardDelete`:
            
        .. code-block:: python
           from pyews import UserConfiguration
           from pyews import DeleteItem
           
           userConfig = UserConfiguration(
               'first.last@company.com',
               'mypassword123'
           )
        
           messageId = 'AAMkAGZjOTlkOWExLTM2MDEtNGI3MS04ZDJiLTllNzgwNDQxMThmMABGAAAAAABdQG8UG7qjTKf0wCVbqLyMBwC6DuFzUH4qRojG/OZVoLCfAAAAAAEMAAC6DuFzUH4qRojG/OZVoLCfAAAu4Y9UAAA='

           deleteItem = DeleteItem(messageId, userConfig, delete_type='HardDelete')
           
    Args:
        messageId (list or str): An email MessageId to delete
        userconfiguration (UserConfiguration): A UserConfiguration object created using the UserConfiguration class
        delete_type (str, optional): Defaults to MoveToDeletedItems. Specify the DeleteType.  Available options are ['HardDelete', 'SoftDelete', 'MoveToDeletedItems']

    Raises:
        SoapAccessDeniedError: Access is denied when attempting to use Exchange Web Services endpoint
        SoapResponseHasError: An error occurred when parsing the SOAP response
        ObjectType: An incorrect object type has been used
    '''

    DELETE_TYPES = ['HardDelete', 'SoftDelete', 'MoveToDeletedItems']

    def __init__(self, messageId, userconfiguration, delete_type=DELETE_TYPES[2]):

        self.messageId = messageId

        super(DeleteItem, self).__init__(userconfiguration)

        if delete_type in self.DELETE_TYPES:
            self.delete_type = delete_type
        else:
            raise DeleteTypeError('You must provide one of the following delete types: %s' % self.DELETE_TYPES)

        self._soap_request = self.soap(self.messageId)
        self.invoke(self._soap_request)
        self.response = self.raw_soap


    @property
    def response(self):
        '''DeleteItem SOAP response dictionary
        
        Returns:
            list: Returns a formatted dictionary of a SOAP response
        '''
        return self._response

    @response.setter
    def response(self, value):
        '''Creates and sets a response object
        
        Args:
            value (str): The raw response from a SOAP request
        '''
        return_list = []
        if value.find('ResponseCode').string != 'NoError':
            for item in value.find_all('DeleteItemResponseMessage'):
                return_list.append({
                    'MessageText': item.MessageText.string,
                    'ResponseCode': item.ResponseCode.string,
                    'DescriptiveLinkKey': item.DescriptiveLinkKey.string
                })
        else:
            return_list.append({
                'MessageText': 'Successfull'
            })
        self._response = return_list      

    def soap(self, item):
        '''Creates the SOAP XML message body

        Args:
            item (str or list): A single or list of message ids to delete

        Returns:
            str: Returns the SOAP XML request body
        '''
        if (self.userconfiguration.impersonation):
            impersonation_header = self.userconfiguration.impersonation.header
        else:
            impersonation_header = ''

        delete_item_soap_element = self._delete_item_soap_string(item)
        return '''<?xml version="1.0" encoding="UTF-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
               xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages">
   <soap:Header>
      <t:RequestServerVersion Version="%s" />
      %s
   </soap:Header>
   <soap:Body >
    <DeleteItem DeleteType="%s" xmlns="http://schemas.microsoft.com/exchange/services/2006/messages">
      <ItemIds>
        %s
      </ItemIds>
    </DeleteItem>
  </soap:Body>
</soap:Envelope>''' % (self.userconfiguration.exchangeVersion, impersonation_header, self.delete_type, delete_item_soap_element)
        

    def _delete_item_soap_string(self, item):
        '''Creates a ItemId XML element from a single or list of items

        Returns:
            str: Returns the ItemId SOAP XML element(s)
        '''
        item_soap_string = ''
        if (isinstance(item, list)):
            for i in item:
                item_soap_string += '''<t:ItemId Id="%s"/>''' % i
        else:
            item_soap_string = '''<t:ItemId Id="%s"/>''' % item
        return item_soap_string



from pyews.utils.exchangeversion import ExchangeVersion
from pyews.utils.exceptions import ExchangeVersionError


class Endpoint(object):

    def __init__(self, exchangeVersion, domain=None):
        self.domain = domain
        self.exchangeVersion = exchangeVersion
        self.endpoint = exchangeVersion

    @property
    def exchangeVersion(self):
        return self._exchangeVersion

    @exchangeVersion.setter
    def exchangeVersion(self, value):
        exchange_ver_list = []
        if isinstance(value, list):
            for ver in value:
                if ExchangeVersion.valid_version(ver):
                    exchange_ver_list.append(ver)
        else:
            exchange_ver_list.append(value)
        self._exchangeVersion = exchange_ver_list

    @property
    def domain(self):
        return self._domain

    @domain.setter
    def domain(self, value):
        self._domain = value

    @property
    def endpoint(self):
        return self._endpoint

    @endpoint.setter
    def endpoint(self, value):
        endpoint_list = []
        if self.domain:
            print(self.domain)
            endpoint_list.append("https://%s/autodiscover/autodiscover.svc" % self.domain)
            endpoint_list.append("https://autodiscover.%s/autodiscover/autodiscover.svc" % self.domain)

        endpoint_list.append('https://outlook.office365.com/autodiscover/autodiscover.svc')
        self._endpoint = endpoint_list



pass


import re
import datetime

SEARCH_QUERY_MAP = {
    'attachment': { 'type': str },
    'subject': { 'type': str },
    'body': { 'type': str },
    'to': { 'type': str },
    'from':{ 'type': str },
    'cc': { 'type': str },
    'bcc': { 'type': str },
    'participants': { 'type': str },
    'category': { 'type': str },
    'importance': { 'type': str },
    'kind': ['contacts','docs','email','externaldata','faxes','im','journals','meetings','microsoftteams','notes','posts','rssfeeds','tasks','voicemail'],
    'sent': { 'type': datetime.datetime },
    'received': { 'type': datetime.datetime },
    'hasattachment': { 'type': bool },
    'isflagged': { 'type': bool },
    'isread': { 'type': bool },
    'size': { 'type': int },
}

RELATIONAL_OPERATORS = ['<', '>', '=', '..']

OPERATORS = ['AND', 'OR', 'NOT']



class SearchQuery(object):
        
    def __init__(self, query):
        SEARCH_QUERY_MAP.values()
        
        split_query = re.split('[:]', query)
        for key, value in SEARCH_QUERY_MAP.items():
            if (key == split_query[0]):
                if (split_query[0] == 'kind'):
                    if (split_query[1] in value):
                        print(split_query[1])
               # elif (split_query[0] == 'sent'):

             #   elif (split_query[0] == 'received'):

              #  elif (split_query[0] == 'hasattachment'):

               # elif (split_query[0] == 'isflagged'):

                #elif (split_query[0] == 'isread'):

                else:
                    if (isinstance(split_query[1], value['type'])):
                        print(split_query)
            #if ()
'''
        valid_hashattachment = ['true', 'false']
        if hasattachment:
            if hasattachment not in valid_hashattachment:
                raise ValueError("results: hasattachment must be one of %r." % valid_hashattachment)

        valid_isread = ['true', 'false']
        if isread:
            if isread not in valid_isread:
                raise ValueError("results: isread must be one of %r." % valid_isread)

        valid_category = ['Blue Category', 'Green Category', 'Orange Category', 'Purple Category', 'Red Category', 'Yellow Category']
        if category:
            if category not in valid_category:
                raise ValueError("results: category must be one of %r." % valid_category)

        valid_kind = ['contacts','docs','email','externaldata','faxes','im','journals','meetings','microsoftteams','notes','posts','rssfeeds','tasks','voicemail']
        if kind:
            if kind not in valid_kind:
                raise ValueError("results: kind must be one of %r." % valid_kind)
'''    


import re

class Credentials(object):
    '''Creates a credential object for communicating with EWS
        
        Example:
            
            Here is a basic example of createing a new Credentials object
            
            .. code-block:: python

               creds = Credentials(
                   'first.last@company.com',
                   'mypassword123'
               )

            You can access your credential properties, including a domain property like this:

            .. code-block:: python
              
               print(creds.email_address)
               print(creds.password)
               print(creds.domain)

        Args:
            email_address (str): The email address used in the EWS SOAP request
            password (str): The password used in the EWS SOAP request
        '''

    def __init__(self, email_address, password):
        
        self.email_address = email_address
        self.password = password
        self.domain = self.email_address

    @property
    def email_address(self):
        return self._email_address

    @email_address.setter
    def email_address(self, value):
        self._email_address = value

    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, value):
        self._password = value

    @property
    def domain(self):
        return self._domain

    @domain.setter
    def domain(self, value):
        '''Splits the domain from an email address
        
        Returns:
            str: Returns the split domain from an email address
        '''
        local, _, domain = value.partition('@')
        self._domain = domain


import requests
from bs4 import BeautifulSoup

from .serviceendpoint import ServiceEndpoint
from pyews.configuration.autodiscover import Autodiscover
from pyews.utils.exchangeversion import ExchangeVersion
from pyews.utils.exceptions import ObjectType, SoapResponseHasError, SoapAccessDeniedError


class GetSearchableMailboxes(ServiceEndpoint):
    '''Child class of ServiceEndpoint that identifies all searchable mailboxes based on the provided UserConfiguration object's permissions
    
    Example:
        To use any service class you must provide a :doc:`../configuration/userconfiguration` object first.
        Like all service classes, you can access formatted properties from the EWS endpoint using the `response` property.

        You can acquire 
            
        .. code-block:: python
           from pyews import UserConfiguration
           from pyews import GetSearchableMailboxes

           userConfig = UserConfiguration(
               'first.last@company.com',
               'mypassword123'
           )

           searchableMailboxes = GetSearchableMailboxes(userconfig)

        If you want to use a property from this object with another class then you can iterate through the list of of mailbox properties.
        For example, if used in conjunction with the :doc:`searchmailboxes` you first need to create a list of mailbox ReferenceIds.

        .. code-block:: python
           
           id_list = []
           for id in searchableMailboxes.response:
               id_list.append(id['ReferenceId'])
           searchResults = SearchMailboxes('subject:"Phishing Email Subject"', userConfig, id_list)


    Args:
        userconfiguration (UserConfiguration): A :doc:`../configuration/userconfiguration` object created using the :doc:`../configuration/userconfiguration` class
    Raises:
        SoapAccessDeniedError: Access is denied when attempting to use Exchange Web Services endpoint
        SoapResponseHasError: An error occurred when parsing the SOAP response
        ObjectType: An incorrect object type has been used
    '''

    def __init__(self, userconfiguration):
    
        super(GetSearchableMailboxes, self).__init__(userconfiguration)

        self._soap_request = self.soap()
        self.invoke(self._soap_request)
        self.response = self.raw_soap

    @property
    def response(self):
        '''GetSearchableMailboxes SOAP response
        
        Returns:
            list: Returns a formatted list of one or more searchable mailboxes objects
        '''
        return self._response

    @response.setter
    def response(self, value):
        '''Creates and sets a response object

        Args:
            value (str): The raw response from a SOAP request
        '''
        return_list = []
        if value.find('ResponseCode').string == 'NoError':
            for item in value.find_all('SearchableMailbox'):
                return_list.append({
                    'ReferenceId': item.ReferenceId.string,
                    'PrimarySmtpAddress': item.PrimarySmtpAddress.string,
                    'DisplayName': item.DisplayName.string,
                    'IsMembershipGroup': item.IsMembershipGroup.string,
                    'IsExternalMailbox': item.IsExternalMailbox.string,
                    'ExternalEmailAddress': item.ExternalEmailAddress.string,
                    'Guid': item.Guid.string
                })
        self._response = return_list

    def soap(self):
        '''Creates the SOAP XML message body

        Returns:
            str: Returns the SOAP XML request body
        '''
        if (self.userconfiguration.impersonation):
            impersonation_header = self.userconfiguration.impersonation.header
        else:
            impersonation_header = ''

        return '''<?xml version="1.0" encoding="UTF-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
               xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages">
   <soap:Header>
      <t:RequestServerVersion Version="%s" />
      %s
   </soap:Header>
   <soap:Body >
      <m:GetSearchableMailboxes>
         <m:ExpandGroupMembership>true</m:ExpandGroupMembership>
      </m:GetSearchableMailboxes>
   </soap:Body>
</soap:Envelope>''' % (self.userconfiguration.exchangeVersion, impersonation_header)


import logging
from bs4 import BeautifulSoup
import requests

from .credentials import Credentials
from .autodiscover import Autodiscover
from .impersonation import Impersonation

from pyews.service.resolvenames import ResolveNames
from pyews.utils.exceptions import ObjectType, IncorrectParameters, ExchangeVersionError, UserConfigurationError
from pyews.utils.exchangeversion import ExchangeVersion

__LOGGER__ = logging.getLogger(__name__)


class UserConfiguration(object):
    '''UserConfiguration is the main class of pyews.  It is used by all other ServiceEndpoint parent and child classes.  
    This class represents how you authorize communication with all other SOAP requests throughout this package.
        
    Examples:

        The UserConfiguration class is the main class used by all services, including the parent class of services called ServiceEndpoint. 

        A UserConfiguration object contains detailed information about how to communicate to Exchange Web Services, as well as additional properties

        The traditional UserConfiguration object can be created by just passing in a username and password.  This will attempt to connect using Autodiscover and will attempt every version of Exchange.

            .. code-block:: python
             
               userConfig = UserConfiguration(
                   'first.last@company.com',
                   'mypassword123'
               )

        If you the know the Exchange version you want to communicate with you can provide this information:
            
            .. code-block:: python
             
               userConfig = UserConfiguration(
                   'first.last@company.com',
                   'mypassword123',
                   exchangeVersion='Office365'
               )

        If you do not wish to use Autodiscover then you can tell UserConfiguration to not use it by setting autodiscover to False and provide the ewsUrl instead

            .. code-block:: python
             
               userConfig = UserConfiguration(
                   'first.last@company.com',
                   'mypassword123',
                   autodiscover=False,
                   ewsUrl='https://outlook.office365.com/EWS/Exchange.asmx'
               )

        If you would like to use impersonation, you first need to create an Impersonation object and pass that into the UserConfiguration class when creating a user configuration object.

            .. code-block:: python
             
               impersonation = Impersonation(primarysmtpaddress='first.last@company.com')

               userConfig = UserConfiguration(
                   'first.last@company.com',
                   'mypassword123',
                   autodiscover=False,
                   ewsUrl='https://outlook.office365.com/EWS/Exchange.asmx',
                   impersonation=impersonation
               )

    Args:
        username (str): An email address or username that you use to authenticate to Exchange Web Services
        password (str): The password that you use to authenticate to Exchange Web Services
        exchangeVersion (str, optional): Defaults to None. A string representation of the version of Exchange you are connecting to.  
                                            If no version is provided then we will attempt to use Autodiscover to determine this version from all approved versions
        ewsUrl (str, optional): Defaults to None. An alternative Exchange Web Services URL.
        autodiscover (bool, optional): Defaults to True. If you don't want to use Autodiscover then set this to False, but you must provide a ewsUrl.
        impersonation (bool, optional): Defaults to False. If your credentials have impersonation rights then you can set this value to True to impersonate another user
                                                            but you must provide either a principalname, sid, primarysmtpaddress, or smtpaddress to do so.
        principalname ([type], optional): Defaults to None. Only used when impersonation is set to True.  The PrincipalName of the account you want to impersonate
        sid ([type], optional): Defaults to None. Only used when impersonation is set to True.  The SID of the account you want to impersonate
        primarysmtpaddress ([type], optional): Defaults to None. Only used when impersonation is set to True.  The PrimarySmtpAddress of the account you want to impersonate
        smtpaddress ([type], optional): Defaults to None. Only used when impersonation is set to True.  The SmtpAddress of the account you want to impersonate
    
    Raises:
        IncorrectParameters: Provided an incorrect configuration of parameters to this class
    '''
    
    def __init__(self, username, password, exchangeVersion=None, ewsUrl= None, autodiscover=True, impersonation=None):
            
        self.credentials = username, password
        self.exchangeVersion = exchangeVersion
        self.ewsUrl = ewsUrl
        self.impersonation = impersonation

        if autodiscover:
            self.autodiscover = autodiscover
        else:
            if ewsUrl:
                self.raw_soap = ResolveNames(self).response
                self.properties = self.raw_soap
            else:
                raise IncorrectParameters('If you are not using Autodiscover then you must provide a ewsUrl and exchangeVersion.')
        
    @property
    def impersonation(self):
        return self._impersonation

    @impersonation.setter
    def impersonation(self, value):
        if isinstance(value, Impersonation):
            self._impersonation = value
        else:
            self._impersonation = None

    @property
    def credentials(self):
        '''Returns a Credentials object
        
        Returns:
            Credentials: A Credentials object made of your passed in username and password
        '''
        return self._credentials

    @credentials.setter
    def credentials(self, creds):
        '''Sets and creates a Credentials object
        
        Args:
            creds (list): A list that contains a username and password
        
        Raises:
            CredentialsError: Error when attempting to create a Credentials object
        '''
        username, password = creds
        try:
            self._credentials = Credentials(username, password)
        except:
            raise CredentialsError('Unable to create credential object using the provided username and password.')

    @property
    def autodiscover(self):
        '''Returns True if Autodiscover is used and False if it is not
        
        Returns:
            bool: Returns True of False
        '''
        return self._autodiscover

    @autodiscover.setter
    def autodiscover(self, boolean):
        '''Sets Autodiscover value to True or False
        
        Args:
            boolean (str): Sets Autodiscover to True or False
        '''
        if self.exchangeVersion:
            self.raw_soap = Autodiscover(self.credentials,exchangeVersion=self.exchangeVersion).response
            self.properties = self.raw_soap
        else:
            self.raw_soap = Autodiscover(self.credentials).response
            self.properties = self.raw_soap

        self._autodiscover = bool(boolean)

    @property
    def exchangeVersion(self):
        '''The Exchange Version used to connect to EWS

        Returns:
             str: The Exchange Version used to connect to EWS
        '''
        return self._exchangeVersion

    @exchangeVersion.setter
    def exchangeVersion(self, value):
        '''Verifies and sets the ExchangeVersion for the UserConfiguration.
        This value is used in all ServiceEndpoints parent and child classes
        
        Args:
            value (str): A string value that represents the ExchangeVersion wanting to be used
        
        Raises:
            ExchangeVersionError: An error occurred when attempting to verify that the value passed in was a valid ExchangeVersion
        '''
        if ExchangeVersion.valid_version(value):
            if value is 'Office365' or 'Exchange2016':
                self._exchangeVersion = 'Exchange2016'
            else:
                self._exchangeVersion = value
        else:
            self._exchangeVersion = None

    @property
    def ewsUrl(self):
        '''The EWS Url used to connect to EWS, if not using Autodiscover
        
        Returns:
            str: The EWS URL used to connect to EWS, if not using Autodiscover
        '''
        return self._ewsUrl

    @ewsUrl.setter
    def ewsUrl(self, url):
        '''Sets the EWS Url to use if not using Autodiscover
        
        Args:
            url (str): The EWS Url to use if not using Autodiscover
        '''
        self._ewsUrl = url

    @property
    def properties(self):
        '''Returns a dictionary containing all UserConfiguration properties and values
        
        Returns:
            dict: A dictionary containing all UserConfiguration properties and values
        '''
        return_dict = {}
        temp = vars(self)
        for item in temp:
            if not item.startswith('_'):
                return_dict.update({
                    item: temp[item]
                })
        return return_dict

    @properties.setter
    def properties(self, config):
        '''Takes a BeautifulSoup4 object created from the response of either AutoDiscover or ResolveNames to build a dictionary of UserConfiguration properties and values
        
        Args:
            config (BeautifulSoup): A BeautifulSoup object created from the response of either AutoDiscover or ResolveNames
        
        Raises:
            UserConfigurationError: An error occured when create the configuration object.  This is a critical error in pyews.
        '''
        try:
            if config.find('ErrorCode'):
                if config.find('ErrorCode').string == 'NoError':
                    if config.find('GetUserSettingsResponseMessage'):
                        self._parse_autodiscover_properties(config)
            if config.find('ResponseCode'):
                if config.find('ResponseCode').string == 'NoError':
                    if config.find('ResolveNamesResponse'):
                        self._parse_resolvenames_properties(config)
        except:
            __LOGGER__.warning("An error occurred attempting to parse SOAP response from Exchange Web Services.  Unable to create UserConfiguration properties.", exc_info=True)
         

    def _parse_autodiscover_properties(self, config):
        for item in config.find_all('UserSetting'):
            if (item.Name.string == 'CasVersion'):
                self.exchangeVersion = ExchangeVersion(item.Value.string).exchangeVersion
            elif (item.Name.string == 'ExternalEwsUrl'):
                self.ewsUrl = item.Value.string
            else:
                setattr(self, item.Name.string, item.Value.string)

    def _parse_resolvenames_properties(self, config):
        temp = config.find('ServerVersionInfo')
        ver = "%s.%s" % (
            temp['MajorVersion'],
            temp['MinorVersion']
        )
        self.exchangeVersion = ExchangeVersion(ver).exchangeVersion
        for item in config.find('ResolutionSet'):
            for i in item.find('Mailbox'):
                setattr(self, i.name, i.string)
            for i in item.find('Contact').descendants:
                if i.name == 'Entry' and i.string:
                    setattr(self, i['Key'], i.string)
                else:
                    if i.name and i.string:
                        setattr(self, i.name, i.string)

    @property
    def raw_soap(self):
        '''Returns the raw SOAP response
        
        Returns:
            str: Raw SOAP XML response
        '''
        return self._raw_soap

    @raw_soap.setter
    def raw_soap(self, value):
        '''Sets the raw soap response from a SOAP request
        
        Args:
            value (str): The response from a SOAP request
        '''
        self._raw_soap = value


import requests
from bs4 import BeautifulSoup

from .serviceendpoint import ServiceEndpoint
from pyews.utils.exceptions import ObjectType, SoapResponseHasError, SoapAccessDeniedError


class ResolveNames(ServiceEndpoint):
    '''Child class of :doc:`serviceendpoint` that is used to resolve names based on the provided :doc:`../configuration/userconfiguration` object.  This class is used as an alternative to :doc:`../configuration/autodiscover`
        since ResolveNames endpoint is a common endpoint across all versions of Microsoft Exchange & Office 365.
        
        Examples:
            To use any service class you must provide a :doc:`../configuration/userconfiguration` object first.
            Like all service classes, you can access formatted properties from the EWS endpoint using the `response` property.
            
            By passing in a :doc:`../configuration/userconfiguration` object we can 
                
            .. code-block:: python

               userConfig = UserConfiguration(
                   'first.last@company.com',
                   'mypassword123'
               )
        
               messageId = 'AAMkAGZjOTlkOWExLTM2MDEtNGI3MS04ZDJiLTllNzgwNDQxMThmMABGAAAAAABdQG8UG7qjTKf0wCVbqLyMBwC6DuFzUH4qRojG/OZVoLCfAAAAAAEMAAC6DuFzUH4qRojG/OZVoLCfAAAu4Y9UAAA='
               deleteItem = DeleteItem(messageId, userConfig)

        Args:
            userconfiguration (UserConfiguration): A :doc:`../configuration/userconfiguration` object created using the :doc:`../configuration/userconfiguration` class
        
        Raises:
            ObjectType: An incorrect object type has been used
        '''

    def __init__(self, userconfiguration):

        super(ResolveNames, self).__init__(userconfiguration)
        
        self._soap_request = self.soap()
        self.invoke(self._soap_request)
        self.response = self.raw_soap

    @property
    def response(self):
        '''ResolveNames SOAP response
        
        Returns:
            str: Returns the ResolveNames response
        '''
        return self._response

    @response.setter
    def response(self, value):
        '''Creates and sets a response object

        Args:
            value (str): The raw response from a SOAP request
        '''
        self._response = value

    def soap(self):
        '''Creates the SOAP XML message body

        Returns:
            str: Returns the SOAP XML request body
        '''
        return '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:a="http://schemas.microsoft.com/exchange/2010/Autodiscover"      
               xmlns:wsa="http://www.w3.org/2005/08/addressing" 
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"      
               xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Body>
    <ResolveNames xmlns="http://schemas.microsoft.com/exchange/services/2006/messages"
                  xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
                  ReturnFullContactData="true">
      <UnresolvedEntry>%s</UnresolvedEntry>
    </ResolveNames>
  </soap:Body>
</soap:Envelope>
        ''' % self.userconfiguration.credentials.email_address


from bs4 import BeautifulSoup
import requests, re, logging
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from pyews.utils.exceptions import SoapResponseHasError, SoapAccessDeniedError, SoapConnectionError

__LOGGER__ = logging.getLogger(__name__)


class ServiceEndpoint(object):
    
    SOAP_REQUEST_HEADER = {'content-type': 'text/xml; charset=UTF-8'}

    def __init__(self, userconfiguration):
        '''Parent class of all endpoints implemented within pyews
        
        Args:
            userconfiguration (UserConfiguration): A UserConfiguration object created using the UserConfiguration class
        '''

        self.userconfiguration = userconfiguration

        self.results = []

    @property
    def userconfiguration(self):
        '''Returns a UserConfiguration object
        
        Returns:
            UserConfiguration: Returns a UserConfiguration object
        '''
        return self._userconfiguration

    @userconfiguration.setter
    def userconfiguration(self, config):
        '''Sets a UserConfiguration object from a child class
        
        Args:
            config (UserConfiguration): A UserConfiguration object created using the UserConfiguration class
        '''

        # deferring importing of userconfiguration until now.
        # This definitely feels hacky but for now 
        # we are going to do this until we have a better solution

        from ..configuration.userconfiguration import UserConfiguration

        if isinstance(config, UserConfiguration):
            self._userconfiguration = config

    @property
    def raw_soap(self):
        '''Returns the raw SOAP response
        
        Returns:
            str: Raw SOAP XML response
        '''
        return self._raw_soap

    @raw_soap.setter
    def raw_soap(self, value):
        '''Sets the raw soap and response from a SOAP request
        
        Args:
            value (str): The response from a SOAP request
        '''
        self._raw_soap = value

    def invoke(self, soap_request):
        '''Used to invoke an Autodiscover SOAP request
        
        Args:
            soap_request (str): A formatted SOAP XML request body string
            userconfiguration (UserConfiguration): A UserConfiguration object created using the UserConfiguration class

        Raises:
            SoapResponseHasError: Raises an error when unable to parse a SOAP response
        '''
        try:
            response = requests.post(
                url=self.userconfiguration.ewsUrl,
                data=soap_request,
                headers=self.SOAP_REQUEST_HEADER,
                auth=(self.userconfiguration.credentials.email_address, self.userconfiguration.credentials.password),
                verify=False
            )
        except requests.exceptions.RequestException as e:
            __LOGGER__.warning(
                "An {err} occurred connecting to Exchange Web Services: {ep}".format(
                    err=e.__class__.__name__,
                    ep=self.userconfiguration.ewsUrl
                ),
                exc_info=True
            )
            raise SoapConnectionError('Error sending SOAP XML payload to {ep}'.format(ep=self.userconfiguration.ewsUrl))

        parsed_response = BeautifulSoup(response.content, 'xml')
        if parsed_response.find('ResponseCode').string == 'NoError':
            self.raw_soap = parsed_response
            return
        elif parsed_response.find('ResponseCode').string == 'ErrorAccessDenied':
            raise SoapAccessDeniedError('%s' % parsed_response.find('MessageText').string)

        raise SoapResponseHasError('Unable to parse response from {current}' % self.__class__.__name__)


import requests, logging
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from bs4 import BeautifulSoup

from pyews.configuration.endpoint import Endpoint
from pyews.utils.exchangeversion import ExchangeVersion
from pyews.utils.exceptions import IncorrectParameters, ExchangeVersionError, SoapResponseHasError, SoapResponseIsNoneError, SoapConnectionRefused, SoapConnectionError

__LOGGER__ = logging.getLogger(__name__)


class Autodiscover(object):
    '''A class used to connect to Exchange Web Services using the Autodiscover service endpoint
    
    The Autodiscover class can be used with both Office 365 and on-premises Exchange 2010 to 2019.
    Currently, it has been thoroughly tested with Office 365 but not so much with the on-premises versions of Exchange.
    
    Example:
        There two typical methods of using the Autodiscover class.  These behave slightly differently depending on your needs.
            1. If you know the Autodiscover URL for your on-premises or Office 365 Exchange Autodiscover service then you can provide this directly
            
            .. code-block:: python

               Autodiscover(
                   credentialObj, 
                   endpoint='https://outlook.office365.com/autodiscover/autodiscover.svc',
                   exchangeVersion='Office365'
               )

            2. If you do not know the Autodiscover URL then you can set create_endpoint_list to True to have the Autodiscover class attempt to generate URL endpoints for you
            
            .. code-block:: python

               Autodiscover(
                   credentialObj,
                   create_endpoint_list=True
               )

    Args:
        credentials (Credentials): An object created using the Credentials class
        endpoint (str, optional): Defaults to None. If you want to specify a different Autodiscover endpoint then provide the url here
        exchangeVersion (str, optional): Defaults to None. An exchange version string
        create_endpoint_list (bool, optional): Defaults to False. If you want the Autodiscover class to generate a list of endpoints to try based on a users email address
    
    Raises:
        IncorrectParameters: An error occurred by not passing the correct parameters into this class
        ExchangeVersionError: An error occurred when passing in an exchange version that is not supported
        SoapResponseHasError: An error occurred when parsing the SOAP response

    '''

    def __init__(self, credentials, endpoint=None, exchangeVersion=None):

        self.credentials = credentials
        
        if exchangeVersion:
            self.exchangeVersion = exchangeVersion
        else:
            self.exchangeVersion = ExchangeVersion.EXCHANGE_VERSIONS

        if endpoint:
            self.endpoint = endpoint
        else:
            self.endpoint = Endpoint(self.exchangeVersion, domain=self.credentials.domain).endpoint

        _response = self.invoke()
        if _response:
            self.response = _response
        else:
            raise SoapResponseIsNoneError('Unable to determine autodiscover configuration.')

    @property
    def endpoint(self):
        '''The Autodiscover endpoint used when connecting to EWS
        
        Returns:
            :obj:`list` of :obj:`str`: Autodiscover endpoint URL used to connect with EWS
        '''
        return self._endpoint

    @endpoint.setter
    def endpoint(self, value):
        endpoint_list = []
        if isinstance(value, list):
            for item in value:
                endpoint_list.append(item)
        else:
            endpoint_list.append(value)
        self._endpoint = endpoint_list

    @property
    def response(self):
        '''Autodiscover service response
        
        Returns:
            str: Response message from the Autodiscover service
        '''
        return self._response

    @response.setter
    def response(self, value):
        self._response = value

    @property
    def exchangeVersion(self):
        '''The Exchange Version used to connect to the Autodiscover service
        
        Returns:
            str: The Exchange Version used to connect to the Autodiscover service
        '''
        return self._exchangeVersion

    @exchangeVersion.setter
    def exchangeVersion(self, value):
        return_list = []
        if isinstance(value, list):
            for item in value:
                if ExchangeVersion.valid_version(item):
                    if item is 'Office365' or 'Exchange2016':
                        return_list.append('Exchange2016')
                    else:
                        return_list.append(value)
        else:
            if ExchangeVersion.valid_version(value):
                if value is 'Office365' or 'Exchange2016':
                    return_list.append('Exchange2016')
                else:
                    return_list.append(value)

        self._exchangeVersion = return_list

    def invoke(self):
        '''Used to invoke an Autodiscover SOAP request
        
        Returns:
            str: Returns response from SOAP request
        '''
        soap_list = []
        for ver in self.exchangeVersion:
            for endpoint in self.endpoint:
                __LOGGER__.debug("Determining if Exchange Version of {ver} and {ep} is a valid endpoint".format(ver=ver,ep=endpoint))
                try:
                    requests.get(endpoint)
                    soap_request = self._soap_request(endpoint, ver)
                    autodiscover_result = self._send_autodiscover_payload(endpoint, soap_request)
                    __LOGGER__.debug(autodiscover_result)
                    if autodiscover_result:
                        return autodiscover_result
                except requests.RequestException as e:
                    __LOGGER__.debug(
                        "An Error {err} occurred when checking if Exchange Version {ver} and Autodiscover endpoint of {ep} is accessible.".format(
                            err=e.__class__.__name__,
                            ver=ver,
                            ep=endpoint
                        )
                    )
                    continue


    def _send_autodiscover_payload(self, endpoint, soap_body):
        '''Used to send and retrieve response from EWS
        
        Args:
            url (str): Autodiscover URL for requests
            version (str): Exchange version used in SOAP request body
        '''
        headers = {'content-type': 'text/xml'}
        try:
            response = requests.post(
                endpoint,
                data=soap_body,
                headers=headers,
                auth=(self.credentials.email_address, self.credentials.password),
                stream=True
            )
        except requests.exceptions.RequestException as e:
            __LOGGER__.warning(
                "An {err} occurred connecting to autodiscover endpoint: {ep}".format(
                    err=e.__class__.__name__,
                    ep=endpoint
                ),
                exc_info=True
            )

        __LOGGER__.debug('Raw Resposne from Requests is %s' % response.text)
        parsed_response = BeautifulSoup(response.content, 'xml')
        __LOGGER__.debug('Parsed Response is %s' % parsed_response)
        if parsed_response.find('ErrorCode').string == 'NoError':
            return parsed_response
        
        raise SoapResponseHasError('The Autodiscover Parsed Response contains an error')


    def _soap_request(self, url, version):
        '''Builds XML SOAP request
        
        Args:
            url (str): Autodiscover URL for requests
            version (str): Exchange version used in SOAP request body
        '''
        return '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:a="http://schemas.microsoft.com/exchange/2010/Autodiscover"      
               xmlns:wsa="http://www.w3.org/2005/08/addressing" 
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"      
               xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Header>
    <a:RequestedServerVersion>%s</a:RequestedServerVersion>
    <wsa:Action>http://schemas.microsoft.com/exchange/2010/Autodiscover/Autodiscover/GetUserSettings</wsa:Action>
    <wsa:To>%s</wsa:To>
  </soap:Header>
  <soap:Body>
    <a:GetUserSettingsRequestMessage xmlns:a="http://schemas.microsoft.com/exchange/2010/Autodiscover">
      <a:Request>
        <a:Users>
          <a:User>
            <a:Mailbox>%s</a:Mailbox>
          </a:User>
        </a:Users>
        <a:RequestedSettings>
          <a:Setting>InternalEwsUrl</a:Setting>
          <a:Setting>ExternalEwsUrl</a:Setting>
          <a:Setting>UserDisplayName</a:Setting>
          <a:Setting>UserDN</a:Setting>
          <a:Setting>UserDeploymentId</a:Setting>
          <a:Setting>InternalMailboxServer</a:Setting>
          <a:Setting>MailboxDN</a:Setting>
          <a:Setting>ActiveDirectoryServer</a:Setting>
          <a:Setting>CasVersion</a:Setting>
          <a:Setting>EwsSupportedSchemas</a:Setting>
        </a:RequestedSettings>
      </a:Request>
    </a:GetUserSettingsRequestMessage>
  </soap:Body>
</soap:Envelope>''' % (version, url, self.credentials.email_address)




import requests
from bs4 import BeautifulSoup

from .serviceendpoint import ServiceEndpoint
from pyews.utils.exceptions import ObjectType, SoapResponseHasError, SoapAccessDeniedError


class GetInboxRules (ServiceEndpoint):
    '''Child class of doc:`serviceendpoint` that retrieves inbox (mailbox) rules for a specified email address.
    
    Examples:
        To use any service class you must provide a :doc:`../configuration/userconfiguration` object first.
        Like all service classes, you can access formatted properties from the EWS endpoint using the `response` property.
        
        If you want to retrieve the inbox rules for a specific email address you must provide it when creating a GetInboxRules object.
            
        .. code-block:: python
           from pyews import UserConfiguration
           from pyews import GetInboxRules
           
           userConfig = UserConfiguration(
               'first.last@company.com',
               'mypassword123'
           )

           inboxRules = GetInboxRules('first.last@company.com', userConfig)

    Args:
        smtp_address (str): The email address you want to get inbox rules for
        userconfiguration (UserConfiguration): A :doc:`../configuration/userconfiguration` object created using the UserConfiguration class

    Raises:
        SoapAccessDeniedError: Access is denied when attempting to use Exchange Web Services endpoint
        SoapResponseHasError: An error occurred when parsing the SOAP response
        ObjectType: An incorrect object type has been used
    '''
    
    def __init__(self, smtp_address, userconfiguration):

        self.email_address = smtp_address

        super(GetInboxRules, self).__init__(userconfiguration)

        self._soap_request = self.soap(self.email_address)
        self.invoke(self._soap_request)
        self.response = self.raw_soap

    @property
    def response(self):
        '''GetInboxRules SOAP response
        
        Returns:
            list: Returns a formatted list of dictionaries of a SOAP response
        '''
        return self._response

    @response.setter
    def response(self, value):
        '''Creates and sets a response object
        
        Args:
            value (str): The raw response from a SOAP request
        '''
        return_list = []
        if value.find('ResponseCode').string == 'NoError':
            for item in value.find('Rule'):
                return_dict = {}
                if (item.name == 'Conditions'):
                    for child in item.descendants:
                        if (child.name is not None and child.string is not None):
                            return_dict.update({
                                child.name: child.string
                            })
                if (item.name == 'Actions'):
                    for child in item.descendants:
                        if (child.name is not None and child.string is not None):
                            return_dict.update({
                                child.name: child.string
                            })
                else:
                    return_dict.update({
                        item.name: item.string
                    })
                    
                return_list.append(return_dict)
            self._response = return_list


    def soap(self, email_address):
        '''Creates the SOAP XML message body

        Args:
            email_address (str): A single email addresses you want to GetInboxRules for

        Returns:
            str: Returns the SOAP XML request body
        '''
        if (self.userconfiguration.impersonation):
            impersonation_header = self.userconfiguration.impersonation.header
        else:
            impersonation_header = ''

        return '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages"
        xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
        xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Header>
    <t:RequestServerVersion Version="%s" />
    %s
  </soap:Header>
  <soap:Body>
    <m:GetInboxRules>
      <m:MailboxSmtpAddress>%s</m:MailboxSmtpAddress>
    </m:GetInboxRules>
  </soap:Body>
</soap:Envelope>
        ''' % (self.userconfiguration.exchangeVersion, impersonation_header, email_address)


from pyews.utils.logger import setup_logging
setup_logging()

from .configuration import (autodiscover, userconfiguration, credentials, impersonation)
from .service import (deleteitem, getinboxrules, getsearchablemailboxes, resolvenames, searchmailboxes, searchquery, serviceendpoint)
from .utils import (exceptions, exchangeversion, logger)

from .configuration.userconfiguration import UserConfiguration
from .configuration.endpoint import Endpoint
from .configuration.autodiscover import Autodiscover
from .configuration.credentials import Credentials
from .service.serviceendpoint import ServiceEndpoint
from .service.resolvenames import ResolveNames
from .service.deleteitem import DeleteItem
from .service.getinboxrules import GetInboxRules
from .service.getsearchablemailboxes import GetSearchableMailboxes
from .service.searchmailboxes import SearchMailboxes
from .service.searchquery import SearchQuery
from .utils.exchangeversion import ExchangeVersion
from .utils.exceptions import CredentialsError, IncorrectParameters, SoapConnectionError, SoapConnectionRefused, ExchangeVersionError, ObjectType, SearchScopeError, SoapAccessDeniedError, SoapResponseHasError, SoapResponseIsNoneError, DeleteTypeError, UserConfigurationError




class Impersonation(object):
    '''The Impersonation class is used when you want to impersonate a user.  You must access rights to impersonate a specific user within your Exchange environment.
    
    Example:
        Below are examples of the data inputs expected for all parameters.
        
        .. code-block:: python

           Impersonation(principalname='first.last@company.com')
           Impersonation(primarysmtpaddress='first.last@company.com')
           Impersonation(smtpaddress='first.last@company.com')

    Args:
        principalname (str, optional): The PrincipalName of the account you want to impersonate
        sid (str, optional): The SID of the account you want to impersonate
        primarysmtpaddress (str, optional): The PrimarySmtpAddress of the account you want to impersonate
        smtpaddress (bool, optional): The SmtpAddress of the account you want to impersonate
    
    Raises:
        AttributeError: This will raise when you call this class but do not provide at least 1 parameter
    '''

    def __init__(self, principalname=None, sid=None, primarysmtpaddress=None, smtpaddress=None):

        if principalname:
            self.impersonation_type = 'PrincipalName'
            self.impersonation_value = principalname
        elif sid:
            self.impersonation_type = 'SID'
            self.impersonation_value = sid
        elif primarysmtpaddress:
            self.impersonation_type = 'PrimarySmtpAddress'
            self.impersonation_value = primarysmtpaddress
        elif smtpaddress:
            self.impersonation_type = 'SmtpAddress'
            self.impersonation_value = smtpaddress
        else:
            raise AttributeError('By setting impersonation to true you must provide either a PrincipalName, SID, PrimarySmtpAddress, or SmtpAddress')

        self.header = self._create_impersonation_header()

    def _create_impersonation_header(self):
        return '''<soap:Header>
  <t:ExchangeImpersonation>
    <t:ConnectingSID>
      <t:%s>%s</t:%s>
    </t:ConnectingSID>
  </t:ExchangeImpersonation>
</soap:Header>''' % (self.impersonation_type, self.impersonation_value, self.impersonation_type)


from setuptools import setup, find_packages

def parse_requirements(requirement_file):
    with open(requirement_file) as f:
        return f.readlines()

setup(
    name='py-ews',
    version='1.0.6',
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    description='A Python package to interact with both on-premises and Office 365 Exchange Web Services',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    install_requires=parse_requirements('./requirements.txt'),
    keywords=['ews', 'exchange', 'office365', 'email', 'ediscovery', 'swimlane'],
    url='https://github.com/swimlane/pyews',
    author='Swimlane',
    author_email='info@swimlane.com',
    python_requires='>=2.6, !=3.0.*, !=3.1.*, !=3.2.*, <4'
)


class IncorrectParameters(Exception):
    '''Raised when the incorrect configuration of parameters is passed into a Class'''
    pass

class SoapConnectionRefused(Exception):
    '''Raised when a connection is refused from the server'''
    pass

class SoapConnectionError(Exception):
    '''Raised when an error occurs attempting to connect to Exchange Web Services endpoint'''
    pass

class SoapResponseIsNoneError(Exception):
    '''Raised when a SOAP request response is None'''
    pass

class SoapResponseHasError(Exception):
    '''Raised when a SOAP request response contains an error'''
    pass

class SoapAccessDeniedError(Exception):
    '''Raised when a SOAP response message says Access Denied'''
    pass

class ExchangeVersionError(Exception):
    '''Raised when using an Exchange Version that is not supported'''
    pass

class ObjectType(Exception):
    '''Raised when the object type used is not the correct type'''
    pass

class DeleteTypeError(Exception):
    '''Incorrect DeleteType is used'''
    pass

class SearchScopeError(Exception):
    '''Incorrect SearchScope is used'''
    pass

class CredentialsError(Exception):
    '''Unable to create a credential object with the provided input'''
    pass

class UserConfigurationError(Exception):
    '''Unable to create or set a UserConfiguration Configuration property'''
    pass


import requests, re
from bs4 import BeautifulSoup

from .serviceendpoint import ServiceEndpoint
from pyews.utils.exchangeversion import ExchangeVersion
from pyews.utils.exceptions import ObjectType, DeleteTypeError, SoapResponseHasError, SoapAccessDeniedError


class SearchMailboxes(ServiceEndpoint):
    '''Child class of :doc:`serviceendpoint` that is used to search mailboxes based on a search query, :doc:`../configuration/userconfiguration` object, and a single or list of mailbox ReferenceId's.  
        
        Examples:
            To use any service class you must provide a :doc:`../configuration/userconfiguration` object first.
            Like all service classes, you can access formatted properties from the EWS endpoint using the `response` property.
            
            The search query parameter takes a specific format, below are examples of different situations as well as comments that explain that situation:

            .. code-block:: guess
                   
                   # Searching for a keyword in a subject
                   # You can specify the word you are looking for as either `account` or `Account`
                   searchQuery = 'subject:account'

                   # Searching for an exact string in a subject
                   searchQuery = 'subject:"Your Account is about to expire"'

                   # Searching for a url in a email's body
                   searchQuery = 'body:"https://google.com"'

                   # You can combine your search query by adding grouping logical operators
                   # Here is an example of searching for two subject strings and a body string match
                   searchQuery = 'subject:(account OR phishing) AND body:"https://google.com"'

            For more information take a look at Microsoft's documentation for their= `Advanced Query Syntax (AQS) <https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/querystring-querystringtype>`_

            
            By passing in a search_query, :doc:`../configuration/userconfiguration` object, and a mailbox_id we can search that specific mailbox or a list of mailbox referenceIds

            .. code-block:: python
               from pyews import UserConfiguration
               from pyews import SearchMailboxes

               userConfig = UserConfiguration(
                   'first.last@company.com',
                   'mypassword123'
               )
        
               referenceId = '/o=ExchangeLabs/ou=Exchange Administrative Group (FYDIBOHF23SPDLT)/cn=Recipients/cn=5341a4228e8c433ba81b4b4b6d75e100-last.first'
               searchResults = SearchMailboxes('subject:account', userConfig, referenceId)

        Args:
            search_query (str): A EWS QueryString.  More information can be found at https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/querystring-querystringtype
            userconfiguration (UserConfiguration): A :doc:`../configuration/userconfiguration` object created using the :doc:`../configuration/userconfiguration` class
            mailbox_id (str or list): A single or list of mailbox IDs to search.  This mailbox id is a ReferenceId
            search_scope (str, optional): Defaults to 'All'. The search scope for the provided mailbox ids.  The options are ['All', 'PrimaryOnly', 'ArchiveOnly']
        
        Raises:
            ObjectType: An incorrect object type has been used
            SearchScopeError: The provided search scope is not one of the following options: ['All', 'PrimaryOnly', 'ArchiveOnly']
        '''

    def __init__(self, search_query, userconfiguration, mailbox_id, search_scope='All'):

        self.search_query = search_query
        
        super(SearchMailboxes, self).__init__(userconfiguration)

        self.mailbox_list = mailbox_id

        if (search_scope in ['All', 'PrimaryOnly', 'ArchiveOnly']):
            self.search_scope = search_scope
        else:
            raise SearchScopeError('Please use the default SearchScope of All or specify PrimaryOnly or ArchiveOnly')

        self._soap_request = self.soap(self.mailbox_list)
        self.invoke(self._soap_request)
        self.response = self.raw_soap

    @property
    def response(self):
        '''Formatted response from SearchMailboxes endpoint.  A list is returned of all identified messages matching search query.
        
        Returns:
            str: Returns the SearchMailboxes response
        '''
        return self._response

    @response.setter
    def response(self, value):
        '''Creates and sets a response object
        
        Args:
            value (str): The raw response from a SOAP request
        '''
        return_list = []
        if value.find('ResponseCode').string == 'NoError':
            for item in value.find_all('SearchPreviewItem'):
                return_dict = {}
                for i in item:
                    if (i.name == 'Id'):
                        return_dict.update({'MessageId': i['Id']})
                    elif (i.name is not None and i.string is not None):
                        return_dict.update({
                            i.name: i.string
                        })
                return_list.append(return_dict)
            self._response = return_list
              

    def soap(self, mailbox):
        '''Creates the SOAP XML message body

        Args:
            mailbox (str or list): A single or list of email mailbox ReferenceIds to search

        Returns:
            str: Returns the SOAP XML request body
        '''
        if (self.userconfiguration.impersonation):
            impersonation_header = self.userconfiguration.impersonation.header
        else:
            impersonation_header = ''

        mailbox_search_scope = self._mailbox_search_scope(mailbox)

        return '''<?xml version="1.0" encoding="UTF-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
               xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
               xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages">
   <soap:Header>
      <t:RequestServerVersion Version="%s" />
      %s
   </soap:Header>
   <soap:Body >
      <m:SearchMailboxes>
         <m:SearchQueries>
            <t:MailboxQuery>
               <t:Query>%s</t:Query>
               <t:MailboxSearchScopes>
                  %s
               </t:MailboxSearchScopes>
            </t:MailboxQuery>
         </m:SearchQueries>
         <m:ResultType>PreviewOnly</m:ResultType>
      </m:SearchMailboxes>
   </soap:Body>
</soap:Envelope>''' % (self.userconfiguration.exchangeVersion, impersonation_header, self.search_query, mailbox_search_scope)


    def _mailbox_search_scope(self,  mailbox):
        '''Creates a MailboxSearchScope XML element from a single or list of mailbox ReferenceIds

        Returns:
            str: Returns the MailboxSearchScope SOAP XML element(s)
        '''
        mailbox_soap_element = ''
        if (isinstance(mailbox, list)):
            for item in mailbox:
                mailbox_soap_element += '''<t:MailboxSearchScope>
        <t:Mailbox>%s</t:Mailbox>
        <t:SearchScope>%s</t:SearchScope>
    </t:MailboxSearchScope>''' % (item, self.search_scope)
        else:
            mailbox_soap_element = '''<t:MailboxSearchScope>
        <t:Mailbox>%s</t:Mailbox>
        <t:SearchScope>%s</t:SearchScope>
    </t:MailboxSearchScope>''' % (mailbox, self.search_scope)

        return mailbox_soap_element